@@ -56,8 +56,12 @@ extern int totem_config_validate (
 	struct totem_config *totem_config,
 	const char **error_string);
 
-int totem_config_keyread (
+extern int totem_config_keyread (
 	struct totem_config *totem_config,
 	const char **error_string);
 
+extern int totem_config_find_local_addr_in_nodelist(
+	const char *ipaddr_key_prefix,
+	unsigned int *node_pos);
+
 #endif /* TOTEMCONFIG_H_DEFINED */
