@@ -236,8 +236,6 @@ static int fragment_size = 0;
 
 static int fragment_continuation = 0;
 
-static struct iovec iov_delv;
-
 static int totempg_waiting_transack = 0;
 
 struct totempg_group_instance {
@@ -630,6 +628,7 @@ static void totempg_deliver_fn (
 	int start;
 	const char *data;
 	int datasize;
+	struct iovec iov_delv;
 
 	assembly = assembly_ref (nodeid);
 	assert (assembly);
