@@ -92,6 +92,7 @@ struct totemudpu_member {
 	struct list_head list;
 	struct totem_ip_address member;
 	int fd;
+	int active;
 };
 
 struct totemudpu_instance {
@@ -177,6 +178,12 @@ struct totemudpu_instance {
 	struct totem_ip_address token_target;
 
 	int token_socket;
+
+	qb_loop_timer_handle timer_merge_detect_timeout;
+
+	int send_merge_detect_message;
+
+	unsigned int merge_detect_messages_sent_before_timeout;
 };
 
 struct work_item {
@@ -197,6 +204,12 @@ static int totemudpu_create_sending_socket(
 int totemudpu_member_list_rebind_ip (
 	void *udpu_context);
 
+static void totemudpu_start_merge_detect_timeout(
+	void *udpu_context);
+
+static void totemudpu_stop_merge_detect_timeout(
+	void *udpu_context);
+
 static struct totem_ip_address localhost;
 
 static void totemudpu_instance_initialize (struct totemudpu_instance *instance)
@@ -315,7 +328,8 @@ static inline void ucast_sendmsg (
 static inline void mcast_sendmsg (
 	struct totemudpu_instance *instance,
 	const void *msg,
-	unsigned int msg_len)
+	unsigned int msg_len,
+	int only_active)
 {
 	struct msghdr msg_mcast;
 	int res = 0;
@@ -355,6 +369,13 @@ static inline void mcast_sendmsg (
 			struct totemudpu_member,
 			list);
 
+		/*
+		 * Do not send multicast message if message is not "flush", member
+		 * is inactive and timeout for sending merge message didn't expired.
+		 */
+		if (only_active && !member->active && !instance->send_merge_detect_message)
+			continue ;
+
 		totemip_totemip_to_sockaddr_convert(&member->member,
 			instance->totem_interface->ip_port, &sockaddr, &addrlen);
 		msg_mcast.msg_name = &sockaddr;
@@ -387,6 +408,14 @@ static inline void mcast_sendmsg (
 				"sendmsg(mcast) failed (non-critical)");
 		}
 	}
+
+	if (!only_active || instance->send_merge_detect_message) {
+		/*
+		 * Current message was sent to all nodes
+		 */
+		instance->merge_detect_messages_sent_before_timeout++;
+		instance->send_merge_detect_message = 0;
+	}
 }
 
 int totemudpu_finalize (
@@ -401,6 +430,8 @@ int totemudpu_finalize (
 		close (instance->token_socket);
 	}
 
+	totemudpu_stop_merge_detect_timeout(instance);
+
 	return (res);
 }
 
@@ -819,6 +850,8 @@ int totemudpu_initialize (
 		timer_function_netif_check_timeout,
 		&instance->timer_netif_check_timeout);
 
+	totemudpu_start_merge_detect_timeout(instance);
+
 	*udpu_context = instance;
 	return (0);
 }
@@ -889,7 +922,7 @@ int totemudpu_mcast_flush_send (
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
 	int res = 0;
 
-	mcast_sendmsg (instance, msg, msg_len);
+	mcast_sendmsg (instance, msg, msg_len, 0);
 
 	return (res);
 }
@@ -902,7 +935,7 @@ int totemudpu_mcast_noflush_send (
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
 	int res = 0;
 
-	mcast_sendmsg (instance, msg, msg_len);
+	mcast_sendmsg (instance, msg, msg_len, 1);
 
 	return (res);
 }
@@ -919,10 +952,12 @@ extern int totemudpu_iface_check (void *udpu_context)
 
 extern void totemudpu_net_mtu_adjust (void *udpu_context, struct totem_config *totem_config)
 {
-#define UDPIP_HEADER_SIZE (20 + 8) /* 20 bytes for ip 8 bytes for udp */
+
+	assert(totem_config->interface_count > 0);
+
 	totem_config->net_mtu -= crypto_sec_header_size(totem_config->crypto_cipher_type,
 							totem_config->crypto_hash_type) +
-				 UDPIP_HEADER_SIZE;
+				 totemip_udpip_header_size(totem_config->interfaces[0].bindnet.family);
 }
 
 const char *totemudpu_iface_print (void *udpu_context)  {
@@ -1083,12 +1118,16 @@ int totemudpu_member_add (
 	if (new_member == NULL) {
 		return (-1);
 	}
+
+	memset(new_member, 0, sizeof(*new_member));
+
 	log_printf (LOGSYS_LEVEL_NOTICE, "adding new UDPU member {%s}",
 		totemip_print(member));
 	list_init (&new_member->list);
 	list_add_tail (&new_member->list, &instance->member_list);
 	memcpy (&new_member->member, member, sizeof (struct totem_ip_address));
 	new_member->fd = totemudpu_create_sending_socket(udpu_context, member);
+	new_member->active = 0;
 
 	return (0);
 }
@@ -1168,3 +1207,80 @@ int totemudpu_member_list_rebind_ip (
 
 	return (0);
 }
+
+int totemudpu_member_set_active (
+	void *udpu_context,
+	const struct totem_ip_address *member_ip,
+	int active)
+{
+	struct list_head *list;
+	struct totemudpu_member *member;
+	int addr_found = 0;
+
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+
+	/*
+	 * Find the member to set active flag
+	 */
+	for (list = instance->member_list.next; list != &instance->member_list;	list = list->next) {
+		member = list_entry (list, struct totemudpu_member, list);
+
+		if (totemip_compare (member_ip, &member->member) == 0) {
+			log_printf(LOGSYS_LEVEL_DEBUG,
+			    "Marking UDPU member %s %s",
+			    totemip_print(&member->member),
+			    (active ? "active" : "inactive"));
+
+			member->active = active;
+			addr_found = 1;
+
+			break;
+		}
+	}
+
+	if (!addr_found) {
+		log_printf(LOGSYS_LEVEL_DEBUG,
+		    "Can't find UDPU member %s (should be marked as %s)",
+			    totemip_print(member_ip),
+			    (active ? "active" : "inactive"));
+	}
+
+	return (0);
+}
+
+static void timer_function_merge_detect_timeout (
+	void *data)
+{
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)data;
+
+	if (instance->merge_detect_messages_sent_before_timeout == 0) {
+		instance->send_merge_detect_message = 1;
+	}
+
+	instance->merge_detect_messages_sent_before_timeout = 0;
+
+	totemudpu_start_merge_detect_timeout(instance);
+}
+
+static void totemudpu_start_merge_detect_timeout(
+	void *udpu_context)
+{
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+
+	qb_loop_timer_add(instance->totemudpu_poll_handle,
+	    QB_LOOP_MED,
+	    instance->totem_config->merge_timeout * 2 * QB_TIME_NS_IN_MSEC,
+	    (void *)instance,
+	    timer_function_merge_detect_timeout,
+	    &instance->timer_merge_detect_timeout);
+
+}
+
+static void totemudpu_stop_merge_detect_timeout(
+	void *udpu_context)
+{
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+
+	qb_loop_timer_del(instance->totemudpu_poll_handle,
+	    instance->timer_merge_detect_timeout);
+}
