@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2014 Red Hat, Inc.
+ * Copyright (c) 2009-2015 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -49,9 +49,11 @@
 #include <corosync/logsys.h>
 #include <corosync/coroapi.h>
 #include <corosync/icmap.h>
+#include <corosync/votequorum.h>
 #include <corosync/ipc_votequorum.h>
 
 #include "service.h"
+#include "util.h"
 
 LOGSYS_DECLARE_SUBSYS ("VOTEQ");
 
@@ -69,6 +71,7 @@ static struct corosync_api_v1 *corosync_api;
 static char qdevice_name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
 static struct cluster_node *qdevice = NULL;
 static unsigned int qdevice_timeout = VOTEQUORUM_QDEVICE_DEFAULT_TIMEOUT;
+static unsigned int qdevice_sync_timeout = VOTEQUORUM_QDEVICE_DEFAULT_SYNC_TIMEOUT;
 static uint8_t qdevice_can_operate = 1;
 static void *qdevice_reg_conn = NULL;
 static uint8_t qdevice_master_wins = 0;
@@ -78,8 +81,9 @@ static uint8_t two_node = 0;
 static uint8_t wait_for_all = 0;
 static uint8_t wait_for_all_status = 0;
 
-static uint8_t auto_tie_breaker = 0;
+static enum {ATB_NONE, ATB_LOWEST, ATB_HIGHEST, ATB_LIST} auto_tie_breaker = ATB_NONE;
 static int lowest_node_id = -1;
+static int highest_node_id = -1;
 
 #define DEFAULT_LMS_WIN   10000
 static uint8_t last_man_standing = 0;
@@ -146,6 +150,7 @@ static int votequorum_exec_send_quorum_notification(void *conn, uint64_t context
 
 #define VOTEQUORUM_RECONFIG_PARAM_EXPECTED_VOTES 1
 #define VOTEQUORUM_RECONFIG_PARAM_NODE_VOTES     2
+#define VOTEQUORUM_RECONFIG_PARAM_CANCEL_WFA     3
 
 static int votequorum_exec_send_reconfigure(uint8_t param, unsigned int nodeid, uint32_t value);
 
@@ -197,7 +202,11 @@ static uint8_t cluster_is_quorate;
 static struct cluster_node *us;
 static struct list_head cluster_members_list;
 static unsigned int quorum_members[PROCESSOR_COUNT_MAX];
+static unsigned int previous_quorum_members[PROCESSOR_COUNT_MAX];
+static unsigned int atb_nodelist[PROCESSOR_COUNT_MAX];
 static int quorum_members_entries = 0;
+static int previous_quorum_members_entries = 0;
+static int atb_nodelist_entries = 0;
 static struct memb_ring_id quorum_ringid;
 
 /*
@@ -227,6 +236,8 @@ static corosync_timer_handle_t qdevice_timer;
 static int qdevice_timer_set = 0;
 static corosync_timer_handle_t last_man_standing_timer;
 static int last_man_standing_timer_set = 0;
+static int sync_nodeinfo_sent = 0;
+static int sync_wait_for_poll_or_timeout = 0;
 
 /*
  * Service Interfaces required by service_message_handler struct
@@ -303,6 +314,8 @@ static int quorum_lib_init_fn (void *conn);
 
 static int quorum_lib_exit_fn (void *conn);
 
+static void qdevice_timer_fn(void *arg);
+
 static void message_handler_req_lib_votequorum_getinfo (void *conn,
 							const void *message);
 
@@ -537,6 +550,28 @@ static void get_lowest_node_id(void)
 	LEAVE();
 }
 
+static void get_highest_node_id(void)
+{
+	struct cluster_node *node = NULL;
+	struct list_head *tmp;
+
+	ENTER();
+
+	highest_node_id = us->node_id;
+
+	list_iterate(tmp, &cluster_members_list) {
+		node = list_entry(tmp, struct cluster_node, list);
+		if ((node->state == NODESTATE_MEMBER) &&
+		    (node->node_id > highest_node_id)) {
+			highest_node_id = node->node_id;
+		}
+	}
+	log_printf(LOGSYS_LEVEL_DEBUG, "highest node id: %d us: %d", highest_node_id, us->node_id);
+	icmap_set_uint32("runtime.votequorum.highest_node_id", highest_node_id);
+
+	LEAVE();
+}
+
 static int check_low_node_id_partition(void)
 {
 	struct cluster_node *node = NULL;
@@ -557,6 +592,151 @@ static int check_low_node_id_partition(void)
 	return found;
 }
 
+static int check_high_node_id_partition(void)
+{
+	struct cluster_node *node = NULL;
+	struct list_head *tmp;
+	int found = 0;
+
+	ENTER();
+
+	list_iterate(tmp, &cluster_members_list) {
+		node = list_entry(tmp, struct cluster_node, list);
+		if ((node->state == NODESTATE_MEMBER) &&
+		    (node->node_id == highest_node_id)) {
+				found = 1;
+		}
+	}
+
+	LEAVE();
+	return found;
+}
+
+static int is_in_nodelist(int nodeid, unsigned int *members, int entries)
+{
+	int i;
+	ENTER();
+
+	for (i=0; i<entries; i++) {
+		if (nodeid == members[i]) {
+			LEAVE();
+			return 1;
+		}
+	}
+	LEAVE();
+	return 0;
+}
+
+/*
+ * The algorithm for a list of tie-breaker nodes is:
+ * travel the list of nodes in the auto_tie_breaker list,
+ * if the node IS in our current partition, check if the
+ * nodes earlier in the atb list are in the 'previous' partition;
+ * If none are found then we are safe to be quorate, if any are
+ * then we cannot be as we don't know if that node is up or down.
+ * If we don't have a node in the current list we are NOT quorate.
+ * Obviously if we find the first node in the atb list in our
+ * partition then we are quorate.
+ *
+ * Special cases lowest nodeid, and highest nodeid are handled separately.
+ */
+static int check_auto_tie_breaker(void)
+{
+	int i, j;
+	int res;
+	ENTER();
+
+	if (auto_tie_breaker == ATB_LOWEST) {
+		res = check_low_node_id_partition();
+		log_printf(LOGSYS_LEVEL_DEBUG, "ATB_LOWEST decision: %d", res);
+		LEAVE();
+		return res;
+	}
+	if (auto_tie_breaker == ATB_HIGHEST) {
+		res = check_high_node_id_partition();
+		log_printf(LOGSYS_LEVEL_DEBUG, "ATB_HIGHEST decision: %d", res);
+		LEAVE();
+		return res;
+	}
+
+	/* Assume ATB_LIST, we should never be called for ATB_NONE */
+	for (i=0; i < atb_nodelist_entries; i++) {
+		if (is_in_nodelist(atb_nodelist[i], quorum_members, quorum_members_entries)) {
+			/*
+			 * Node is in our partition, if any of its predecessors are
+			 * in the previous quorum partition then it might be in the
+			 * 'other half' (as we've got this far without seeing it here)
+			 * and so we can't be quorate.
+			 */
+			for (j=0; j<i; j++) {
+				if (is_in_nodelist(atb_nodelist[j], previous_quorum_members, previous_quorum_members_entries)) {
+					log_printf(LOGSYS_LEVEL_DEBUG, "ATB_LIST found node %d in previous partition but not here, quorum denied", atb_nodelist[j]);
+					LEAVE();
+					return 0;
+				}
+			}
+
+			/*
+			 * None of the other list nodes were in the previous partition, if there
+			 * are enough votes, we can be quorate
+			 */
+			log_printf(LOGSYS_LEVEL_DEBUG, "ATB_LIST found node %d in current partition, we can be quorate", atb_nodelist[i]);
+			LEAVE();
+			return 1;
+		}
+	}
+	log_printf(LOGSYS_LEVEL_DEBUG, "ATB_LIST found no list nodes in current partition, we cannot be quorate");
+	LEAVE();
+	return 0;
+}
+
+/*
+ * atb_string can be either:
+ *   'lowest'
+ *   'highest'
+ *   a list of nodeids
+ */
+static void parse_atb_string(char *atb_string)
+{
+	char *ptr;
+	long num;
+
+	ENTER();
+	auto_tie_breaker = ATB_NONE;
+
+	if (!strcmp(atb_string, "lowest"))
+		auto_tie_breaker = ATB_LOWEST;
+
+	if (!strcmp(atb_string, "highest"))
+		auto_tie_breaker = ATB_HIGHEST;
+
+	if (atoi(atb_string)) {
+
+		atb_nodelist_entries = 0;
+		ptr = atb_string;
+		do {
+			num = strtol(ptr, &ptr, 10);
+			if (num) {
+				log_printf(LOGSYS_LEVEL_DEBUG, "ATB nodelist[%d] = %d", atb_nodelist_entries, num);
+				atb_nodelist[atb_nodelist_entries++] = num;
+			}
+		} while (num);
+
+		if (atb_nodelist_entries) {
+			auto_tie_breaker = ATB_LIST;
+		}
+	}
+	icmap_set_uint32("runtime.votequorum.atb_type", auto_tie_breaker);
+	log_printf(LOGSYS_LEVEL_DEBUG, "ATB type = %d", auto_tie_breaker);
+
+	/* Make sure we got something */
+	if (auto_tie_breaker == ATB_NONE) {
+		log_printf(LOGSYS_LEVEL_WARNING, "auto_tie_breaker_nodes is not valid. It must be 'lowest', 'highest' or a space-separated list of node IDs. auto_tie_breaker is disabled");
+		auto_tie_breaker = ATB_NONE;
+	}
+	LEAVE();
+}
+
 static int check_qdevice_master(void)
 {
 	struct cluster_node *node = NULL;
@@ -606,7 +786,7 @@ static int load_ev_tracking_barrier(void)
 
 	ENTER();
 
-	snprintf(filename, sizeof(filename) - 1, LOCALSTATEDIR "/lib/corosync/ev_tracking");
+	snprintf(filename, sizeof(filename) - 1, "%s/ev_tracking", get_run_dir());
 
 	ev_tracking_fd = open(filename, O_RDWR, 0700);
 	if (ev_tracking_fd != -1) {
@@ -827,11 +1007,15 @@ static void are_we_quorate(unsigned int total_votes)
 	} else {
 		quorate = 1;
 		get_lowest_node_id();
+		get_highest_node_id();
 	}
 
-	if ((auto_tie_breaker) &&
+	if ((auto_tie_breaker != ATB_NONE) &&
+	    /* Must be a half (or half-1) split */
 	    (total_votes == (us->expected_votes / 2)) &&
-	    (check_low_node_id_partition() == 1)) {
+	    /* If the 'other' partition in a split might have quorum then we can't run ATB */
+	    (previous_quorum_members_entries - quorum_members_entries < quorum) &&
+	    (check_auto_tie_breaker() == 1)) {
 		quorate = 1;
 	}
 
@@ -1018,6 +1202,9 @@ static int votequorum_qdevice_is_configured(uint32_t *qdevice_votes)
 			if (icmap_get_uint32("quorum.device.timeout", &qdevice_timeout) != CS_OK) {
 				qdevice_timeout = VOTEQUORUM_QDEVICE_DEFAULT_TIMEOUT;
 			}
+			if (icmap_get_uint32("quorum.device.sync_timeout", &qdevice_sync_timeout) != CS_OK) {
+				qdevice_sync_timeout = VOTEQUORUM_QDEVICE_DEFAULT_SYNC_TIMEOUT;
+			}
 			update_qdevice_can_operate(1);
 			ret = 1;
 		}
@@ -1038,7 +1225,9 @@ static char *votequorum_readconfig(int runtime)
 	uint32_t node_votes = 0, qdevice_votes = 0;
 	uint32_t node_expected_votes = 0, expected_votes = 0;
 	uint32_t node_count = 0;
+	uint8_t atb = 0;
 	int have_nodelist, have_qdevice;
+	char *atb_string = NULL;
 	char *error = NULL;
 
 	ENTER();
@@ -1098,10 +1287,29 @@ static char *votequorum_readconfig(int runtime)
 
 		icmap_get_uint8("quorum.allow_downscale", &allow_downscale);
 		icmap_get_uint8("quorum.wait_for_all", &wait_for_all);
-		icmap_get_uint8("quorum.auto_tie_breaker", &auto_tie_breaker);
 		icmap_get_uint8("quorum.last_man_standing", &last_man_standing);
 		icmap_get_uint32("quorum.last_man_standing_window", &last_man_standing_window);
 		icmap_get_uint8("quorum.expected_votes_tracking", &ev_tracking);
+		icmap_get_uint8("quorum.auto_tie_breaker", &atb);
+		icmap_get_string("quorum.auto_tie_breaker_node", &atb_string);
+
+		/* auto_tie_breaker defaults to LOWEST */
+		if (atb) {
+		    auto_tie_breaker = ATB_LOWEST;
+		    icmap_set_uint32("runtime.votequorum.atb_type", auto_tie_breaker);
+		}
+		else {
+			auto_tie_breaker = ATB_NONE;
+			if (atb_string) {
+				log_printf(LOGSYS_LEVEL_WARNING,
+					   "auto_tie_breaker_node: is meaningless if auto_tie_breaker is set to 0");
+			}
+		}
+
+		if (atb && atb_string) {
+			parse_atb_string(atb_string);
+		}
+		free(atb_string);
 
 		/* allow_downscale requires ev_tracking */
 		if (allow_downscale) {
@@ -1118,6 +1326,42 @@ static char *votequorum_readconfig(int runtime)
 
 	}
 
+	/* two_node and auto_tie_breaker are not compatible as two_node uses
+	 * a fence race to decide quorum whereas ATB decides based on node id
+	 */
+	if (two_node && auto_tie_breaker != ATB_NONE) {
+	        log_printf(LOGSYS_LEVEL_CRIT, "two_node and auto_tie_breaker are both specified but are not compatible.");
+		log_printf(LOGSYS_LEVEL_CRIT, "two_node has been disabled, please fix your corosync.conf");
+		two_node = 0;
+	}
+
+	/* If ATB is set and the cluster has an odd number of nodes then wait_for_all needs
+	 * to be set so that an isolated half+1 without the tie breaker node
+	 * does not have quorum on reboot.
+	 */
+	if ((auto_tie_breaker != ATB_NONE) && (node_expected_votes % 2) &&
+	    (!wait_for_all)) {
+		if (last_man_standing) {
+			/* if LMS is set too, it's a fatal configuration error. We can't dictate to the user what
+			 *  they might want so we'll just quit.
+			 */
+			log_printf(LOGSYS_LEVEL_CRIT, "auto_tie_breaker is set, the cluster has an odd number of nodes\n");
+			log_printf(LOGSYS_LEVEL_CRIT, "and last_man_standing is also set. With this situation a better\n");
+			log_printf(LOGSYS_LEVEL_CRIT, "solution would be to disable LMS, leave ATB enabled, and also\n");
+			log_printf(LOGSYS_LEVEL_CRIT, "enable wait_for_all (mandatory for ATB in odd-numbered clusters).\n");
+			log_printf(LOGSYS_LEVEL_CRIT, "Due to this ambiguity, corosync will fail to start. Please fix your corosync.conf\n");
+			error = (char *)"configuration error: auto_tie_breaker & last_man_standing not available in odd sized cluster";
+			goto out;
+		}
+		else {
+			log_printf(LOGSYS_LEVEL_CRIT, "auto_tie_breaker is set and the cluster has an odd number of nodes.\n");
+			log_printf(LOGSYS_LEVEL_CRIT, "wait_for_all needs to be set for this configuration but it is missing\n");
+			log_printf(LOGSYS_LEVEL_CRIT, "Therefore auto_tie_breaker has been disabled. Please fix your corosync.conf\n");
+			auto_tie_breaker = ATB_NONE;
+			icmap_set_uint32("runtime.votequorum.atb_type", auto_tie_breaker);
+		}
+	}
+
 	/*
 	 * quorum device is not compatible with last_man_standing and auto_tie_breaker
 	 * neither lms or atb can be set at runtime, so there is no need to check for
@@ -1136,7 +1380,7 @@ static char *votequorum_readconfig(int runtime)
 		}
 	}
 
-	if ((have_qdevice) && (auto_tie_breaker)) {
+	if ((have_qdevice) && (auto_tie_breaker != ATB_NONE)) {
 		if (!runtime) {
 			error = (char *)"configuration error: quorum.device is not compatible with auto_tie_breaker";
 			goto out;
@@ -1287,9 +1531,28 @@ static void votequorum_refresh_config(
 	void *user_data)
 {
 	int old_votes, old_expected_votes;
+	uint8_t reloading;
+	uint8_t cancel_wfa;
 
 	ENTER();
 
+	/*
+	 * If a full reload is in progress then don't do anything until it's done and
+	 * can reconfigure it all atomically
+	 */
+	if (icmap_get_uint8("config.totemconfig_reload_in_progress", &reloading) == CS_OK && reloading) {
+		return ;
+	}
+
+	icmap_get_uint8("quorum.cancel_wait_for_all", &cancel_wfa);
+	if (strcmp(key_name, "quorum.cancel_wait_for_all") == 0 &&
+	    cancel_wfa >= 1) {
+	        icmap_set_uint8("quorum.cancel_wait_for_all", 0);
+		votequorum_exec_send_reconfigure(VOTEQUORUM_RECONFIG_PARAM_CANCEL_WFA,
+						 us->node_id, 0);
+		return;
+	}
+
 	old_votes = us->votes;
 	old_expected_votes = us->expected_votes;
 
@@ -1319,6 +1582,7 @@ static void votequorum_exec_add_config_notification(void)
 {
 	icmap_track_t icmap_track_nodelist = NULL;
 	icmap_track_t icmap_track_quorum = NULL;
+	icmap_track_t icmap_track_reload = NULL;
 
 	ENTER();
 
@@ -1334,6 +1598,12 @@ static void votequorum_exec_add_config_notification(void)
 		NULL,
 		&icmap_track_quorum);
 
+	icmap_track_add("config.totemconfig_reload_in_progress",
+		ICMAP_TRACK_ADD | ICMAP_TRACK_MODIFY,
+		votequorum_refresh_config,
+		NULL,
+		&icmap_track_reload);
+
 	LEAVE();
 }
 
@@ -1471,6 +1741,8 @@ static int votequorum_exec_send_quorum_notification(void *conn, uint64_t context
 	res_lib_votequorum_notification = (struct res_lib_votequorum_notification *)&buf;
 	res_lib_votequorum_notification->quorate = cluster_is_quorate;
 	res_lib_votequorum_notification->node_list_entries = cluster_members;
+	res_lib_votequorum_notification->ring_id.nodeid = quorum_ringid.rep.nodeid;
+	res_lib_votequorum_notification->ring_id.seq = quorum_ringid.seq;
 	res_lib_votequorum_notification->context = context;
 	list_iterate(tmp, &cluster_members_list) {
 		node = list_entry(tmp, struct cluster_node, list);
@@ -1853,6 +2125,14 @@ static void message_handler_req_exec_votequorum_reconfigure (
 		recalculate_quorum(1, 0);  /* Allow decrease */
 		break;
 
+	case VOTEQUORUM_RECONFIG_PARAM_CANCEL_WFA:
+	        update_wait_for_all_status(0);
+		log_printf(LOGSYS_LEVEL_INFO, "wait_for_all_status reset by user on node %d.",
+			   req_exec_quorum_reconfigure->nodeid);
+		recalculate_quorum(0, 0);
+
+	        break;
+
 	}
 
 	LEAVE();
@@ -1973,6 +2253,8 @@ static void votequorum_sync_init (
 	ENTER();
 
 	sync_in_progress = 1;
+	sync_nodeinfo_sent = 0;
+	sync_wait_for_poll_or_timeout = 0;
 
 	if (member_list_entries > 1) {
 		us->flags &= ~NODE_FLAGS_FIRST;
@@ -2003,7 +2285,7 @@ static void votequorum_sync_init (
 
 	if (last_man_standing) {
 		if (((member_list_entries >= quorum) && (left_nodes)) ||
-		    ((member_list_entries <= quorum) && (auto_tie_breaker) && (check_low_node_id_partition() == 1))) {
+		    ((member_list_entries <= quorum) && (auto_tie_breaker != ATB_NONE) && (check_low_node_id_partition() == 1))) {
 			if (last_man_standing_timer_set) {
 				corosync_api->timer_delete(last_man_standing_timer);
 				last_man_standing_timer_set = 0;
@@ -2015,21 +2297,53 @@ static void votequorum_sync_init (
 		}
 	}
 
+	memcpy(previous_quorum_members, quorum_members, sizeof(unsigned int) * quorum_members_entries);
+	previous_quorum_members_entries = quorum_members_entries;
+
 	memcpy(quorum_members, member_list, sizeof(unsigned int) * member_list_entries);
 	quorum_members_entries = member_list_entries;
 	memcpy(&quorum_ringid, ring_id, sizeof(*ring_id));
 
+	if (us->flags & NODE_FLAGS_QDEVICE_REGISTERED && us->flags & NODE_FLAGS_QDEVICE_ALIVE) {
+		/*
+		 * Reset poll timer. Sync waiting is interrupted on valid qdevice poll or after timeout
+		 */
+		if (qdevice_timer_set) {
+			corosync_api->timer_delete(qdevice_timer);
+		}
+		corosync_api->timer_add_duration((unsigned long long)qdevice_sync_timeout*1000000, qdevice,
+						 qdevice_timer_fn, &qdevice_timer);
+		qdevice_timer_set = 1;
+		sync_wait_for_poll_or_timeout = 1;
+
+		log_printf(LOGSYS_LEVEL_INFO, "waiting for quorum device %s poll (but maximum for %u ms)",
+			qdevice_name, qdevice_sync_timeout);
+	}
+
 	LEAVE();
 }
 
 static int votequorum_sync_process (void)
 {
-	votequorum_exec_send_nodeinfo(us->node_id);
-	votequorum_exec_send_nodeinfo(VOTEQUORUM_QDEVICE_NODEID);
-	if (strlen(qdevice_name)) {
-		votequorum_exec_send_qdevice_reg(VOTEQUORUM_QDEVICE_OPERATION_REGISTER,
-						 qdevice_name);
+
+	if (!sync_nodeinfo_sent) {
+		votequorum_exec_send_nodeinfo(us->node_id);
+		votequorum_exec_send_nodeinfo(VOTEQUORUM_QDEVICE_NODEID);
+		if (strlen(qdevice_name)) {
+			votequorum_exec_send_qdevice_reg(VOTEQUORUM_QDEVICE_OPERATION_REGISTER,
+							 qdevice_name);
+		}
+		sync_nodeinfo_sent = 1;
 	}
+
+	if (us->flags & NODE_FLAGS_QDEVICE_REGISTERED && sync_wait_for_poll_or_timeout) {
+		/*
+		 * Waiting for qdevice to poll with new ringid or timeout
+		 */
+
+		return (-1);
+	}
+
 	return 0;
 }
 
@@ -2124,6 +2438,7 @@ static void qdevice_timer_fn(void *arg)
 	votequorum_exec_send_nodeinfo(us->node_id);
 
 	qdevice_timer_set = 0;
+	sync_wait_for_poll_or_timeout = 0;
 
 	LEAVE();
 }
@@ -2205,7 +2520,7 @@ static void message_handler_req_lib_votequorum_getinfo (void *conn, const void *
 		if (last_man_standing) {
 			res_lib_votequorum_getinfo.flags |= VOTEQUORUM_INFO_LAST_MAN_STANDING;
 		}
-		if (auto_tie_breaker) {
+		if (auto_tie_breaker != ATB_NONE) {
 			res_lib_votequorum_getinfo.flags |= VOTEQUORUM_INFO_AUTO_TIE_BREAKER;
 		}
 		if (allow_downscale) {
@@ -2331,8 +2646,10 @@ static void message_handler_req_lib_votequorum_trackstart (void *conn,
 	const struct req_lib_votequorum_trackstart *req_lib_votequorum_trackstart = message;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 	struct quorum_pd *quorum_pd = (struct quorum_pd *)corosync_api->ipc_private_data_get (conn);
+	cs_error_t error = CS_OK;
 
 	ENTER();
+
 	/*
 	 * If an immediate listing of the current cluster membership
 	 * is requested, generate membership list
@@ -2343,6 +2660,11 @@ static void message_handler_req_lib_votequorum_trackstart (void *conn,
 		votequorum_exec_send_quorum_notification(conn, req_lib_votequorum_trackstart->context);
 	}
 
+	if (quorum_pd->tracking_enabled) {
+		error = CS_ERR_EXIST;
+		goto response_send;
+	}
+
 	/*
 	 * Record requests for tracking
 	 */
@@ -2356,9 +2678,10 @@ static void message_handler_req_lib_votequorum_trackstart (void *conn,
 		list_add (&quorum_pd->list, &trackers_list);
 	}
 
+response_send:
 	res_lib_votequorum_status.header.size = sizeof(res_lib_votequorum_status);
 	res_lib_votequorum_status.header.id = MESSAGE_RES_VOTEQUORUM_STATUS;
-	res_lib_votequorum_status.header.error = CS_OK;
+	res_lib_votequorum_status.header.error = error;
 	corosync_api->ipc_response_send(conn, &res_lib_votequorum_status, sizeof(res_lib_votequorum_status));
 
 	LEAVE();
@@ -2463,6 +2786,7 @@ static void message_handler_req_lib_votequorum_qdevice_unregister (void *conn,
 		if (qdevice_timer_set) {
 			corosync_api->timer_delete(qdevice_timer);
 			qdevice_timer_set = 0;
+			sync_wait_for_poll_or_timeout = 0;
 		}
 		us->flags &= ~NODE_FLAGS_QDEVICE_REGISTERED;
 		us->flags &= ~NODE_FLAGS_QDEVICE_ALIVE;
@@ -2529,6 +2853,15 @@ static void message_handler_req_lib_votequorum_qdevice_poll (void *conn,
 	}
 
 	if (us->flags & NODE_FLAGS_QDEVICE_REGISTERED) {
+		if (!(req_lib_votequorum_qdevice_poll->ring_id.nodeid == quorum_ringid.rep.nodeid &&
+		      req_lib_votequorum_qdevice_poll->ring_id.seq == quorum_ringid.seq)) {
+			log_printf(LOGSYS_LEVEL_DEBUG, "Received poll ring id (%u.%"PRIu64") != last sync "
+			    "ring id (%u.%"PRIu64"). Ignoring poll call.",
+			    req_lib_votequorum_qdevice_poll->ring_id.nodeid, req_lib_votequorum_qdevice_poll->ring_id.seq,
+			    quorum_ringid.rep.nodeid, quorum_ringid.seq);
+			error = CS_ERR_MESSAGE_ERROR;
+			goto out;
+		}
 		if (strncmp(req_lib_votequorum_qdevice_poll->name, qdevice_name, VOTEQUORUM_QDEVICE_MAX_NAME_LEN)) {
 			error = CS_ERR_INVALID_PARAM;
 			goto out;
@@ -2556,6 +2889,7 @@ static void message_handler_req_lib_votequorum_qdevice_poll (void *conn,
 		corosync_api->timer_add_duration((unsigned long long)qdevice_timeout*1000000, qdevice,
 						 qdevice_timer_fn, &qdevice_timer);
 		qdevice_timer_set = 1;
+		sync_wait_for_poll_or_timeout = 0;
 	} else {
 		error = CS_ERR_NOT_EXIST;
 	}
